@@ -12,7 +12,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # attribute_selection.py
-# Copyright (C) 2014 Fracpete (pythonwekawrapper at gmail dot com)
+# Copyright (C) 2014-2015 Fracpete (pythonwekawrapper at gmail dot com)
 
 import javabridge
 import logging
@@ -20,9 +20,8 @@ import argparse
 import os
 import sys
 import weka.core.jvm as jvm
-import weka.core.utils as utils
 import weka.core.types as arrays
-from weka.core.classes import JavaObject
+from weka.core.classes import JavaObject, join_options
 from weka.core.classes import OptionHandler
 from weka.core.capabilities import Capabilities
 from weka.core.dataset import Instances, Instance
@@ -36,7 +35,7 @@ class ASSearch(OptionHandler):
     Wrapper class for attribute selection search algorithm.
     """
 
-    def __init__(self, classname=None, jobject=None, options=None):
+    def __init__(self, classname="weka.attributeSelection.BestFirst", jobject=None, options=None):
         """
         Initializes the specified search algorithm using either the classname or the supplied JB_Object.
         :param classname: the classname of the search algorithms
@@ -75,7 +74,7 @@ class ASEvaluation(OptionHandler):
     Wrapper class for attribute selection evaluation algorithm.
     """
 
-    def __init__(self, classname=None, jobject=None, options=None):
+    def __init__(self, classname="weka.attributeSelection.CfsSubsetEval", jobject=None, options=None):
         """
         Initializes the specified search algorithm using either the classname or the supplied JB_Object.
         :param classname: the classname of the search algorithms
@@ -321,7 +320,7 @@ def main():
 
     jvm.start(jars, max_heap_size=parsed.heap, packages=True)
 
-    logger.debug("Commandline: " + utils.join_options(sys.argv[1:]))
+    logger.debug("Commandline: " + join_options(sys.argv[1:]))
 
     try:
         evaluation = ASEvaluation(classname=parsed.evaluator)
