@@ -20,10 +20,10 @@ import javabridge
 import logging
 import argparse
 import weka.core.jvm as jvm
-import weka.core.utils as wutils
 import weka.core.types as arrays
+import weka.core.classes as classes
 from numpy import *
-from weka.core.classes import JavaObject
+from weka.core.classes import JavaObject, join_options
 from weka.core.classes import OptionHandler
 from weka.core.classes import Random
 from weka.core.capabilities import Capabilities
@@ -39,7 +39,7 @@ class Classifier(OptionHandler):
     Wrapper class for classifiers.
     """
 
-    def __init__(self, classname=None, jobject=None, options=None):
+    def __init__(self, classname="weka.classifiers.rules.ZeroR", jobject=None, options=None):
         """
         Initializes the specified classifier using either the classname or the supplied JB_Object.
         :param classname: the classname of the classifier
@@ -84,7 +84,7 @@ class Classifier(OptionHandler):
         if self.is_updateable:
             javabridge.call(self.jobject, "updateClassifier", "(Lweka/core/Instance;)V", inst.jobject)
         else:
-            logger.critical(wutils.get_classname(self.jobject) + " is not updateable!")
+            logger.critical(classes.get_classname(self.jobject) + " is not updateable!")
 
     def classify_instance(self, inst):
         """
@@ -170,6 +170,7 @@ class SingleClassifierEnhancer(Classifier):
     def classifier(self):
         """
         Returns the base classifier.
+        ;return: the base classifier
         :rtype: Classifier
         """
         return Classifier(jobject=javabridge.call(self.jobject, "getClassifier", "()Lweka/classifiers/Classifier;"))
@@ -179,6 +180,7 @@ class SingleClassifierEnhancer(Classifier):
         """
         Sets the base classifier.
         :param classifier: the base classifier to use
+        :type classifier: Classifier
         """
         javabridge.call(self.jobject, "setClassifier", "(Lweka/classifiers/Classifier;)V", classifier.jobject)
 
@@ -341,7 +343,7 @@ class Kernel(OptionHandler):
         :type inst1: Instance
         """
         jinst1 = None
-        if not inst1 is None:
+        if inst1 is not None:
             jinst1 = inst1.jobject
         return javabridge.call(self.jobject, "eval", "(IILweka/core/Instance;)D", id1, id2, jinst1)
 
@@ -383,7 +385,7 @@ class KernelClassifier(Classifier):
                 "weka/classifiers/KernelHelper", "hasKernelProperty",
                 "(Ljava/lang/Object;)Z",
                 jobject):
-            raise Exception("Does not handle a kernel: " + wutils.get_classname(jobject))
+            raise Exception("Does not handle a kernel: " + classes.get_classname(jobject))
         super(KernelClassifier, self).__init__(classname=classname, jobject=jobject, options=options)
 
     @property
@@ -541,7 +543,7 @@ class CostMatrix(JavaObject):
         :param num_classes: the number of classes
         :type num_classes: int
         """
-        if not matrx is None:
+        if matrx is not None:
             if isinstance(matrx, CostMatrix):
                 jobject = javabridge.make_instance(
                     "weka/classifiers/CostMatrix", "(Lweka/classifiers/CostMatrix;)V", matrx.jobject)
@@ -738,7 +740,8 @@ class CostMatrix(JavaObject):
         """
         return CostMatrix(
             matrx=javabridge.static_call(
-                "weka/classifiers/CostMatrix", "parseMatlab", "(Ljava/lang/String;)Lweka/classifiers/CostMatrix;", matlab))
+                "weka/classifiers/CostMatrix", "parseMatlab",
+                "(Ljava/lang/String;)Lweka/classifiers/CostMatrix;", matlab))
 
 
 class Evaluation(JavaObject):
@@ -804,7 +807,7 @@ class Evaluation(JavaObject):
         :param output: the output generator to use
         :type output: PredictionOutput
         """
-        if not rnd is None:
+        if rnd is not None:
             data.randomize(rnd)
         train_size = int(round(data.num_instances * percentage / 100))
         test_size = data.num_instances - train_size
@@ -829,6 +832,7 @@ class Evaluation(JavaObject):
         if output is None:
             generator = []
         else:
+            output.header = data
             generator = [output.jobject]
         cls = javabridge.call(
             self.jobject, "evaluateModel",
@@ -1491,7 +1495,7 @@ class PredictionOutput(OptionHandler):
     Must be derived from weka.classifiers.evaluation.output.prediction.AbstractOutput
     """
 
-    def __init__(self, classname=None, jobject=None, options=None):
+    def __init__(self, classname="weka.classifiers.evaluation.output.prediction.PlainText", jobject=None, options=None):
         """
         Initializes the specified output generator using either the classname or the supplied JB_Object.
         :param classname: the classname of the generator
@@ -1706,7 +1710,7 @@ def main():
 
     jvm.start(jars, max_heap_size=parsed.heap, packages=True)
 
-    logger.debug("Commandline: " + wutils.join_options(sys.argv[1:]))
+    logger.debug("Commandline: " + join_options(sys.argv[1:]))
 
     try:
         classifier = Classifier(classname=parsed.classifier)
