@@ -20,8 +20,8 @@ import os
 import sys
 import argparse
 import weka.core.jvm as jvm
-import weka.core.utils as utils
-from weka.core.classes import JavaObject
+import weka.core.classes as classes
+from weka.core.classes import JavaObject, join_options
 from weka.core.classes import OptionHandler
 from weka.core.classes import Random
 from weka.core.capabilities import Capabilities
@@ -36,7 +36,7 @@ class Clusterer(OptionHandler):
     Wrapper class for clusterers.
     """
 
-    def __init__(self, classname=None, jobject=None, options=None):
+    def __init__(self, classname="weka.clusterers.SimpleKMeans", jobject=None, options=None):
         """
         Initializes the specified clusterer using either the classname or the supplied JB_Object.
         :param classname: the classname of the clusterer
@@ -81,7 +81,7 @@ class Clusterer(OptionHandler):
         if self.is_updateable:
             javabridge.call(self.jobject, "updateClusterer", "(Lweka/core/Instance;)V", inst.jobject)
         else:
-            logger.critical(utils.get_classname(self.jobject) + " is not updateable!")
+            logger.critical(classes.get_classname(self.jobject) + " is not updateable!")
 
     def update_finished(self):
         """
@@ -90,7 +90,7 @@ class Clusterer(OptionHandler):
         if self.is_updateable:
             javabridge.call(self.jobject, "updateFinished", "()V")
         else:
-            logger.critical(utils.get_classname(self.jobject) + " is not updateable!")
+            logger.critical(classes.get_classname(self.jobject) + " is not updateable!")
 
     def cluster_instance(self, inst):
         """
@@ -146,6 +146,20 @@ class Clusterer(OptionHandler):
         else:
             return None
 
+    @classmethod
+    def make_copy(cls, clusterer):
+        """
+        Creates a copy of the clusterer.
+        :param clusterer: the clustererto copy
+        :type clusterer: Clusterer
+        :return: the copy of the clusterer
+        :rtype: Clusterer
+        """
+        return Clusterer(
+            jobject=javabridge.static_call(
+                "weka/clusterers/AbstractClusterer", "makeCopy",
+                "(Lweka/clusterers/Clusterer;)Lweka/clusterers/Clusterer;", clusterer.jobject))
+
 
 class SingleClustererEnhancer(Clusterer):
     """
@@ -387,7 +401,7 @@ def main():
 
     jvm.start(jars, max_heap_size=parsed.heap, packages=True)
 
-    logger.debug("Commandline: " + utils.join_options(sys.argv[1:]))
+    logger.debug("Commandline: " + join_options(sys.argv[1:]))
 
     try:
         clusterer = Clusterer(classname=parsed.clusterer)
