@@ -14,13 +14,343 @@
 # classes.py
 # Copyright (C) 2014-2015 Fracpete (pythonwekawrapper at gmail dot com)
 
+import inspect
+import json
+import logging
+import re
 import types
 import javabridge
 from javabridge import JWrapper, JClassWrapper
 from javabridge.jutil import JavaException
+from weka.core import types as arrays
 
 
-class JavaObject(object):
+def get_class(classname):
+    """
+    Returns the class object associated with the dot-notation classname.
+    Taken from here: http://stackoverflow.com/a/452981
+    :param classname: the classname
+    :type classname: str
+    :return: the class object
+    :rtype: object
+    """
+    parts = classname.split('.')
+    module = ".".join(parts[:-1])
+    m = __import__(module)
+    for comp in parts[1:]:
+        m = getattr(m, comp)
+    return m
+
+
+def get_jclass(classname):
+    """
+    Returns the Java class object associated with the dot-notation classname.
+    :param classname: the classname
+    :type classname: str
+    :return: the class object
+    :rtype: JB_Object
+    """
+    return javabridge.class_for_name(classname=classname)
+
+
+def get_classname(obj):
+    """
+    Returns the classname of the JB_Object, Python class or object.
+    :param obj: the java object or Python class/object to get the classname for
+    :type obj: object
+    :return: the classname
+    :rtype: str
+    """
+    if isinstance(obj, javabridge.JB_Object):
+        cls = javabridge.call(obj, "getClass", "()Ljava/lang/Class;")
+        return javabridge.call(cls, "getName", "()Ljava/lang/String;")
+    elif inspect.isclass(obj):
+        return obj.__module__ + "." + obj.__name__
+    else:
+        return get_classname(obj.__class__)
+
+
+class Stoppable(object):
+    """
+    Classes that can be stopped.
+    """
+
+    def is_stopped(self):
+        """
+        Returns whether the object has been stopped.
+        :return: whether stopped
+        :rtype: bool
+        """
+        raise Exception("Not implemented!")
+
+    def stop_execution(self):
+        """
+        Triggers the stopping of the object.
+        """
+        raise Exception("Not implemented!")
+
+
+from_dict_handlers = {}
+"""
+The methods that handle the restoration from a JSON dictionary, stored under their 'type'.
+"""
+
+
+def register_dict_handler(typestr, handler):
+    """
+    Registers a handler for restoring an object from a JSON dictionary.
+    :param typestr: the type of the object
+    :type typestr: str
+    :param handler: the method
+    """
+    global from_dict_handlers
+    from_dict_handlers[typestr] = handler
+
+
+def deregister_dict_handler(typestr):
+    """
+    Deregisters a handler for restoring an object from a JSON dictionary.
+    :param typestr: the type of the object
+    :type typestr: str
+    """
+    global from_dict_handlers
+    del from_dict_handlers[typestr]
+
+
+def has_dict_handler(typestr):
+    """
+    Returns the handler for restoring an object from a JSON dictionary.
+    :param typestr: the type of the object
+    :type typestr: str
+    :return: the handler, None if not available
+    """
+    global from_dict_handlers
+    return typestr in from_dict_handlers
+
+
+def get_dict_handler(typestr):
+    """
+    Returns the handler for restoring an object from a JSON dictionary.
+    :param typestr: the type of the object
+    :type typestr: str
+    :return: the handler, None if not available
+    """
+    global from_dict_handlers
+    return from_dict_handlers[str(typestr)]
+
+
+class JSONObject(object):
+    """
+    Ancestor for classes that can be represented as JSON and restored from JSON.
+    """
+
+    def to_dict(self):
+        """
+        Returns a dictionary that represents this object, to be used for JSONification.
+        :return: the object dictionary
+        :rtype: dict
+        """
+        raise Exception("Not implemented!")
+
+    @classmethod
+    def from_dict(cls, d):
+        """
+        Restores an object state from a dictionary, used in de-JSONification.
+        :param d: the object dictionary
+        :type d: dict
+        :return: the object
+        :rtype: object
+        """
+        raise Exception("Not implemented!")
+
+    def to_json(self):
+        """
+        Returns the options as JSON.
+        :return: the object as string
+        :rtype: str
+        """
+        return json.dumps(self.to_dict(), sort_keys=True, indent=2, separators=(',', ': '))
+
+    @classmethod
+    def from_json(cls, s):
+        """
+        Restores the object from the given JSON.
+        :param s: the JSON string to parse
+        :type s: str
+        :return: the
+        """
+        d = json.loads(s)
+        return get_dict_handler(d["type"])(d)
+
+    def shallow_copy(self):
+        """
+        Returns a shallow copy of itself.
+        :return: the copy
+        :rtype: object
+        """
+        return self.from_json(self.to_json())
+
+
+class Configurable(JSONObject):
+    """
+    The ancestor for all actors.
+    """
+
+    def __init__(self, config=None):
+        """
+        Initializes the object.
+        :param config: the dictionary with the options (str -> object).
+        :type config: dict
+        """
+        self._logger = None
+        self._help = {}
+        self._config = self.fix_config({})
+        if config is not None:
+            self.config = config
+        if not has_dict_handler("Configurable"):
+            register_dict_handler("Configurable", Configurable.from_dict)
+
+    def __repr__(self):
+        """
+        Returns Python code for instantiating the object.
+        :return: the representation
+        :rtype: str
+        """
+        return \
+            self.__class__.__module__ + "." + self.__class__.__name__ \
+            + "(config=" + str(self.config) + ")"
+
+    def description(self):
+        """
+        Returns a description of the object.
+        :return: the description
+        :rtype: str
+        """
+        raise Exception("Not implemented!")
+
+    def fix_config(self, options):
+        """
+        Fixes the options, if necessary. I.e., it adds all required elements to the dictionary.
+        :param options: the options to fix
+        :type options: dict
+        :return: the (potentially) fixed options
+        :rtype: dict
+        """
+        return options
+
+    @property
+    def config(self):
+        """
+        Obtains the currently set options of the actor.
+        :return: the options
+        :rtype: dict
+        """
+        return self._config
+
+    @config.setter
+    def config(self, options):
+        """
+        Sets the options of the actor.
+        :param options: the options
+        :type options: dict
+        """
+        self._config = self.fix_config(options)
+
+    def to_dict(self):
+        """
+        Returns a dictionary that represents this object, to be used for JSONification.
+        :return: the object dictionary
+        :rtype: dict
+        """
+        result = {}
+        result["type"] = "Configurable"
+        result["class"] = get_classname(self)
+        result["config"] = {}
+        for k in self._config:
+            v = self._config[k]
+            if isinstance(v, JSONObject):
+                result["config"][k] = v.to_dict()
+            else:
+                result["config"][k] = v
+        return result
+
+    @classmethod
+    def from_dict(cls, d):
+        """
+        Restores its state from a dictionary, used in de-JSONification.
+        :param d: the object dictionary
+        :type d: dict
+        """
+        conf = {}
+        for k in d["config"]:
+            v = d["config"][k]
+            if isinstance(v, dict):
+                conf[str(k)] = get_dict_handler(d["config"]["type"])(v)
+            else:
+                conf[str(k)] = v
+        return get_class(str(d["class"]))(config=conf)
+
+    def new_logger(self):
+        """
+        Returns a new logger instance.
+        :return: the logger instance
+        :rtype: logger
+        """
+        return logging.getLogger(get_classname(self))
+
+    @property
+    def logger(self):
+        """
+        Returns the logger object.
+        :return: the logger
+        :rtype: logger
+        """
+        if self._logger is None:
+            self._logger = self.new_logger()
+        return self._logger
+
+    @property
+    def help(self):
+        """
+        Obtains the help information per option for this actor.
+        :return: the help
+        :rtype: dict
+        """
+        return self._help
+
+    def generate_help(self):
+        """
+        Generates a help string for this actor.
+        :return: the help string
+        :rtype: str
+        """
+        result = []
+        result.append(self.__class__.__name__)
+        result.append(re.sub(r'.', '=', self.__class__.__name__))
+        result.append("")
+        result.append("DESCRIPTION")
+        result.append(self.description())
+        result.append("")
+        result.append("OPTIONS")
+        opts = self.config.keys()
+        opts.sort()
+        for opt in opts:
+            result.append(opt)
+            helpstr = self.help[opt]
+            if helpstr is None:
+                helpstr = "-missing help-"
+            result.append("\t" + helpstr)
+            result.append("")
+        return '\n'.join(result)
+
+    def print_help(self):
+        """
+        Prints a help string for this actor to stdout.
+        """
+        print(self.generate_help())
+
+
+class JavaObject(JSONObject):
     """
     Basic Java object.
     """
@@ -34,6 +364,8 @@ class JavaObject(object):
         if jobject is None:
             raise Exception("No Java object supplied!")
         self.jobject = jobject
+        if not has_dict_handler("JavaObject"):
+            register_dict_handler("JavaObject", JavaObject.from_dict)
 
     def __str__(self):
         """
@@ -42,6 +374,13 @@ class JavaObject(object):
         """
         return javabridge.to_string(self.jobject)
 
+    def __repr__(self):
+        """
+        Just calls the toString() method.
+        :rtype: str
+        """
+        return javabridge.to_string(self.jobject)
+
     def __unicode__(self):
         """
         Just calls the toString() method.
@@ -121,6 +460,37 @@ class JavaObject(object):
         """
         return JClassWrapper(javabridge.call(self.jobject, "getClass", "()Ljava/lang/Class;"))
 
+    @property
+    def is_serializable(self):
+        """
+        Returns true if the object is serialiable.
+        :return: true if serializable
+        :rtype: bool
+        """
+        return JavaObject.check_type(self.jobject, "java.io.Serializable")
+
+    def to_dict(self):
+        """
+        Returns a dictionary that represents this object, to be used for JSONification.
+        :return: the object dictionary
+        :rtype: dict
+        """
+        result = {}
+        result["type"] = "JavaObject"
+        result["class"] = self.classname
+        return result
+
+    @classmethod
+    def from_dict(cls, d):
+        """
+        Restores an object state from a dictionary, used in de-JSONification.
+        :param d: the object dictionary
+        :type d: dict
+        :return: the object
+        :rtype: object
+        """
+        return JavaObject(cls.new_instance(d["class"]))
+
     @classmethod
     def check_type(cls, jobject, intf_or_class, jni_intf_or_class=None):
         """
@@ -152,7 +522,7 @@ class JavaObject(object):
         :type jni_intf_or_class: str
         """
         if not cls.check_type(jobject, intf_or_class, jni_intf_or_class):
-            raise TypeError("Object does not implement or subclass " + intf_or_class + "!")
+            raise TypeError("Object does not implement or subclass " + intf_or_class + ": " + get_classname(jobject))
 
     @classmethod
     def new_instance(cls, classname, jni_classname=None):
@@ -314,7 +684,6 @@ class Enum(JavaObject):
         """
         if jobject is None:
             enumclass = javabridge.class_for_name(classname=enum)
-            enum = enum.replace(".", "/")
             jobject = javabridge.static_call(
                 "java/lang/Enum", "valueOf",
                 "(Ljava/lang/Class;Ljava/lang/String;)Ljava/lang/Enum;",
@@ -391,7 +760,58 @@ class Random(JavaObject):
         return javabridge.call(self.jobject, "nextDouble", "()D")
 
 
-class OptionHandler(JavaObject):
+class Option(JavaObject):
+    """
+    Wrapper for the weka.core.Option class.
+    """
+
+    def __init__(self, jobject):
+        """
+        Initializes the wrapper with the specified option object.
+        :param jobject: the java object to wrap
+        :type jobject: JB_Object
+        """
+        super(Option, self).__init__(jobject)
+        Option.enforce_type(jobject, "weka.core.Option")
+
+    @property
+    def name(self):
+        """
+        Returns the name of the option.
+        :return: the name
+        :rtype: str
+        """
+        return javabridge.call(self.jobject, "name", "()Ljava/lang/String;")
+
+    @property
+    def description(self):
+        """
+        Returns the description of the option.
+        :return: the description
+        :rtype: str
+        """
+        return javabridge.call(self.jobject, "description", "()Ljava/lang/String;")
+
+    @property
+    def synopsis(self):
+        """
+        Returns the synopsis of the option.
+        :return: the synopsis
+        :rtype: str
+        """
+        return javabridge.call(self.jobject, "synopsis", "()Ljava/lang/String;")
+
+    @property
+    def num_arguments(self):
+        """
+        Returns the synopsis of the option.
+        :return: the synopsis
+        :rtype: str
+        """
+        return javabridge.call(self.jobject, "numArguments", "()I")
+
+
+class OptionHandler(JavaObject, Configurable):
     """
     Ancestor for option-handling classes. 
     Classes should implement the weka.core.OptionHandler interface to have any effect.
@@ -407,9 +827,16 @@ class OptionHandler(JavaObject):
         """
         super(OptionHandler, self).__init__(jobject)
         self.is_optionhandler = OptionHandler.check_type(jobject, "weka.core.OptionHandler")
-        if (not options is None) and (len(options) > 0):
+        if (options is not None) and (len(options) > 0):
             self.options = options
-        
+        # we have to manually instantiate some objects, since multiple inheritance doesn't call
+        # Configurable constructor
+        self._logger = None
+        self._help = {}
+        self._config = self.fix_config({})
+        if not has_dict_handler("OptionHandler"):
+            register_dict_handler("OptionHandler", OptionHandler.from_dict)
+
     def global_info(self):
         """
         Returns the globalInfo() result, None if not available.
@@ -420,6 +847,14 @@ class OptionHandler(JavaObject):
         except JavaException:
             return None
 
+    def description(self):
+        """
+        Returns a description of the object.
+        :return: the description
+        :rtype: str
+        """
+        return self.global_info()
+
     @property
     def options(self):
         """
@@ -453,6 +888,31 @@ class OptionHandler(JavaObject):
             "(Ljava/lang/Object;)Ljava/lang/String;",
             self.jobject)
 
+    def to_help(self):
+        """
+        Returns a string that contains the 'global_info' text and the options.
+        :return: the generated help string
+        :rtype: str
+        """
+        result = []
+        result.append(self.classname)
+        result.append("=" * len(self.classname))
+        result.append("")
+        result.append("DESCRIPTION")
+        result.append("")
+        result.append(self.global_info())
+        result.append("")
+        result.append("OPTIONS")
+        result.append("")
+        options = javabridge.call(self.jobject, "listOptions", "()Ljava/util/Enumeration;")
+        enum = javabridge.get_enumeration_wrapper(options)
+        while enum.hasMoreElements():
+            opt = Option(enum.nextElement())
+            result.append(opt.synopsis)
+            result.append(opt.description)
+            result.append("")
+        return '\n'.join(result)
+
     def __str__(self):
         """
         Calls the toString() method of the java object.
@@ -461,6 +921,30 @@ class OptionHandler(JavaObject):
         """
         return javabridge.to_string(self.jobject)
 
+    def to_dict(self):
+        """
+        Returns a dictionary that represents this object, to be used for JSONification.
+        :return: the object dictionary
+        :rtype: dict
+        """
+        result = super(OptionHandler, self).to_dict()
+        result["type"] = "OptionHandler"
+        result["options"] = join_options(self.options)
+        return result
+
+    @classmethod
+    def from_dict(cls, d):
+        """
+        Restores an object state from a dictionary, used in de-JSONification.
+        :param d: the object dictionary
+        :type d: dict
+        :return: the object
+        :rtype: object
+        """
+        result = OptionHandler(cls.new_instance(d["class"]))
+        result.options = split_options(d["options"])
+        return result
+
 
 class SingleIndex(JavaObject):
     """
@@ -796,3 +1280,67 @@ class SelectedTag(JavaObject):
         for i in xrange(length):
             result.append(Tag(javabridge.get_env().get_string(wrapped[i])))
         return result
+
+
+def join_options(options):
+    """
+    Turns the list of options back into a single commandline string.
+    :param options: the list of options to process
+    :type options: list
+    :return: the combined options
+    :rtype: str
+    """
+    return javabridge.static_call(
+        "Lweka/core/Utils;", "joinOptions",
+        "([Ljava/lang/String;)Ljava/lang/String;",
+        options)
+
+
+def split_options(cmdline):
+    """
+    Splits the commandline into a list of options.
+    :param cmdline: the commandline string to split into individual options
+    :type cmdline: str
+    :return: the split list of commandline options
+    :rtype: list
+    """
+    return arrays.string_array_to_list(
+        javabridge.static_call(
+            "Lweka/core/Utils;", "splitOptions",
+            "(Ljava/lang/String;)[Ljava/lang/String;",
+            cmdline))
+
+
+def to_commandline(optionhandler):
+    """
+    Generates a commandline string from the OptionHandler instance.
+    :param optionhandler: the OptionHandler instance to turn into a commandline
+    :type optionhandler: OptionHandler
+    :return: the commandline string
+    :rtype: str
+    """
+    return javabridge.static_call(
+        "Lweka/core/Utils;", "toCommandLine",
+        "(Ljava/lang/Object;)Ljava/lang/String;",
+        optionhandler.jobject)
+
+
+def from_commandline(cmdline, classname=None):
+    """
+    Creates an OptionHandler based on the provided commandline string.
+    :param cmdline: the commandline string to use
+    :type cmdline: str
+    :param classname: the classname of the wrapper to return other than OptionHandler (in dot-notation)
+    :type classname: str
+    :return: the generated option handler instance
+    :rtype: object
+    """
+    params = split_options(cmdline)
+    cls = params[0]
+    params = params[1:]
+    handler = OptionHandler(jobject=javabridge.make_instance(cls.replace(".", "/"), "()V"), options=params)
+    if classname is None:
+        return handler
+    else:
+        c = get_class(classname)
+        return c(jobject=handler.jobject)
\ No newline at end of file
