@@ -12,7 +12,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # dataset.py
-# Copyright (C) 2014 Fracpete (pythonwekawrapper at gmail dot com)
+# Copyright (C) 2014-2015 Fracpete (pythonwekawrapper at gmail dot com)
 
 import javabridge
 import logging
@@ -66,6 +66,15 @@ class Instances(JavaObject):
         """
         return javabridge.call(self.jobject, "relationName", "()Ljava/lang/String;")
 
+    @relationname.setter
+    def relationname(self, value):
+        """
+        Sets the name of the dataset.
+        :param value: the name
+        :type value: str
+        """
+        javabridge.call(self.jobject, "setRelationName", "(Ljava/lang/String;)V", value)
+
     @property
     def num_attributes(self):
         """
@@ -165,7 +174,7 @@ class Instances(JavaObject):
 
     def has_class(self):
         """
-        Returns whether a class attribute is set(convenience method).
+        Returns whether a class attribute is set (convenience method).
         :return: whether a class attribute is currently set
         :rtype: bool
         """
@@ -544,6 +553,14 @@ class Instance(JavaObject):
         """
         return self.__class_index()
 
+    def has_class(self):
+        """
+        Returns whether a class attribute is set (convenience method).
+        :return: whether a class attribute is currently set
+        :rtype: bool
+        """
+        return self.class_index != -1
+
     def set_value(self, index, value):
         """
         Sets the internal value at the specified position (0-based).
@@ -651,7 +668,7 @@ class Instance(JavaObject):
     def create_instance(cls, values, classname="weka.core.DenseInstance", weight=1.0):
         """
         Creates a new instance.
-        :param values: the float values (internal format) to use (numpy array)
+        :param values: the float values (internal format) to use, numpy array or list.
         :type values: ndarray or list
         :param classname: the classname of the instance (eg weka.core.DenseInstance).
         :type classname: str
@@ -660,9 +677,42 @@ class Instance(JavaObject):
         """
         jni_classname = classname.replace(".", "/")
         if type(values) is list:
+            for i in xrange(len(values)):
+                values[i] = float(values[i])
             values = numpy.array(values)
         return Instance(
-            javabridge.make_instance(jni_classname, "(D[D)V", weight, javabridge.get_env().make_double_array(values)))
+            javabridge.make_instance(
+                jni_classname, "(D[D)V",
+                weight, javabridge.get_env().make_double_array(values)))
+
+    @classmethod
+    def create_sparse_instance(cls, values, max_values, classname="weka.core.SparseInstance", weight=1.0):
+        """
+        Creates a new sparse instance.
+        :param values: the list of tuples (0-based index and internal format float). The indices of the
+                       tuples must be in ascending order and "max_values" must be set to the maximum
+                       number of attributes in the dataset.
+        :type values: list
+        :param max_values: the maximum number of attributes
+        :type max_values: int
+        :param classname: the classname of the instance (eg weka.core.SparseInstance).
+        :type classname: str
+        :param weight: the weight of the instance
+        :type weight: float
+        """
+        jni_classname = classname.replace(".", "/")
+        indices = []
+        vals = []
+        for (i, v) in values:
+            indices.append(i)
+            vals.append(float(v))
+        indices = numpy.array(indices, dtype=numpy.int32)
+        vals = numpy.array(vals)
+        return Instance(
+            javabridge.make_instance(
+                jni_classname, "(D[D[II)V",
+                weight, javabridge.get_env().make_double_array(vals),
+                javabridge.get_env().make_int_array(indices), max_values))
 
 
 class Attribute(JavaObject):
@@ -719,7 +769,7 @@ class Attribute(JavaObject):
         :return: the 0-based index
         :rtype: int
         """
-        return javabridge.call(self.jobject, "indexOf", "(Ljava/lang/String;)I", label)
+        return javabridge.call(self.jobject, "indexOfValue", "(Ljava/lang/String;)I", label)
 
     def value(self, index):
         """
@@ -885,7 +935,7 @@ class Attribute(JavaObject):
         :return: the index
         :rtype: int
         """
-        return javabridge.call(self.jobject, "addStringValue", "(S)I", s)
+        return javabridge.call(self.jobject, "addStringValue", "(Ljava/lang/String;)I", s)
 
     def add_relation(self, instances):
         """
