@@ -12,7 +12,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 # datagenerators.py
-# Copyright (C) 2014 Fracpete (pythonwekawrapper at gmail dot com)
+# Copyright (C) 2014-2015 Fracpete (pythonwekawrapper at gmail dot com)
 
 import javabridge
 import logging
@@ -20,8 +20,8 @@ import os
 import sys
 import argparse
 import weka.core.jvm as jvm
-import weka.core.utils as utils
-from weka.core.classes import OptionHandler
+import weka.core.classes as classes
+from weka.core.classes import OptionHandler, join_options, to_commandline, from_commandline
 from weka.core.dataset import Instances, Instance
 
 # logging setup
@@ -33,7 +33,7 @@ class DataGenerator(OptionHandler):
     Wrapper class for datagenerators.
     """
 
-    def __init__(self, classname=None, jobject=None, options=None):
+    def __init__(self, classname="weka.datagenerators.classifiers.classification.Agrawal", jobject=None, options=None):
         """
         Initializes the specified datagenerator using either the classname or the supplied JB_Object.
         :param classname: the classname of the datagenerator
@@ -152,6 +152,18 @@ class DataGenerator(OptionHandler):
             "(Lweka/datagenerators/DataGenerator;[Ljava/lang/String;)V",
             generator.jobject, args)
 
+    @classmethod
+    def make_copy(cls, generator):
+        """
+        Creates a copy of the generator.
+        :param generator: the generator to copy
+        :type generator: DataGenerator
+        :return: the copy of the generator
+        :rtype: DataGenerator
+        """
+        return from_commandline(
+            to_commandline(generator), classname=classes.get_classname(DataGenerator()))
+
 
 def main():
     """
@@ -171,7 +183,7 @@ def main():
 
     jvm.start(jars, max_heap_size=parsed.heap, packages=True)
 
-    logger.debug("Commandline: " + utils.join_options(sys.argv[1:]))
+    logger.debug("Commandline: " + join_options(sys.argv[1:]))
 
     try:
         generator = DataGenerator(classname=parsed.datagenerator)
