@@ -20,13 +20,15 @@ import os
 import sys
 import argparse
 import weka.core.jvm as jvm
-import weka.core.utils as utils
-from weka.core.classes import OptionHandler
+from weka.core.classes import OptionHandler, join_options
 from weka.core.capabilities import Capabilities
 from weka.core.converters import Loader
 from weka.core.converters import Saver
 from weka.core.dataset import Instances
 from weka.core.dataset import Instance
+from weka.core.stemmers import Stemmer
+from weka.core.stopwords import Stopwords
+from weka.core.tokenizers import Tokenizer
 
 # logging setup
 logger = logging.getLogger("weka.filters")
@@ -37,7 +39,7 @@ class Filter(OptionHandler):
     Wrapper class for filters.
     """
 
-    def __init__(self, classname=None, jobject=None, options=None):
+    def __init__(self, classname="weka.filters.AllFilter", jobject=None, options=None):
         """
         Initializes the specified filter using either the classname or the supplied JB_Object.
         :param classname: the classname of the filter
@@ -52,6 +54,7 @@ class Filter(OptionHandler):
         self.enforce_type(jobject, "weka.filters.Filter")
         super(Filter, self).__init__(jobject=jobject, options=options)
         self.__input = javabridge.make_call(self.jobject, "input", "(Lweka/core/Instance;)Z")
+        self.__batchfinished = javabridge.make_call(self.jobject, "batchFinished", "()Z")
         self.__output = javabridge.make_call(self.jobject, "output", "()Lweka/core/Instance;")
         self.__outputformat = javabridge.make_call(self.jobject, "getOutputFormat", "()Lweka/core/Instances;")
 
@@ -76,9 +79,19 @@ class Filter(OptionHandler):
         Inputs the Instance.
         :param inst: the instance to filter
         :type inst: Instance
+        :return: True if filtered can be collected from output
+        :rtype: bool
         """
         return self.__input(inst.jobject)
 
+    def batch_finished(self):
+        """
+        Signals the filter that the batch of data has finished.
+        :return: True if instances can be collected from the output
+        :rtype: bool
+        """
+        return self.__batchfinished()
+
     def outputformat(self):
         """
         Returns the output format.
@@ -124,6 +137,20 @@ class Filter(OptionHandler):
                 "(Lweka/core/Instances;Lweka/filters/Filter;)Lweka/core/Instances;",
                 data.jobject, self.jobject))
 
+    @classmethod
+    def make_copy(cls, flter):
+        """
+        Creates a copy of the filter.
+        :param flter: the filter to copy
+        :type flter: Filter
+        :return: the copy of the filter
+        :rtype: Filter
+        """
+        return Filter(
+            jobject=javabridge.static_call(
+                "weka/filters/Filter", "makeCopy",
+                "(Lweka/filters/Filter;)Lweka/filters/Filter;", flter.jobject))
+
 
 class MultiFilter(Filter):
     """
@@ -171,6 +198,86 @@ class MultiFilter(Filter):
         javabridge.call(self.jobject, "setFilters", "([Lweka/filters/Filter;)V", obj)
 
 
+class StringToWordVector(Filter):
+    """
+    Wrapper class for weka.filters.unsupervised.attribute.StringToWordVector.
+    """
+
+    def __init__(self, jobject=None, options=None):
+        """
+        Initializes the StringToWordVector instance using either creating new instance or using the supplied JB_Object.
+        :param jobject: the JB_Object to use
+        :type jobject: JB_Object
+        :param options: list of commandline options
+        :type options: list
+        """
+        if jobject is None:
+            classname = "weka.filters.unsupervised.attribute.StringToWordVector"
+            jobject = StringToWordVector.new_instance(classname)
+        self.enforce_type(jobject, "weka.filters.unsupervised.attribute.StringToWordVector")
+        super(StringToWordVector, self).__init__(jobject=jobject, options=options)
+
+    @property
+    def stemmer(self):
+        """
+        Returns the stemmer.
+        :return: the stemmer
+        :rtype: Stemmer
+        """
+        return Stemmer(
+            jobject=javabridge.call(self.jobject, "getStemmer", "()Lweka/core/stemmers/Stemmer;"))
+
+    @stemmer.setter
+    def stemmer(self, stemmer):
+        """
+        Sets the stemmer.
+        :param stemmer: the stemmer to use
+        :type stemmer: Stemmer
+        """
+        javabridge.call(
+            self.jobject, "setStemmer", "(Lweka/core/stemmers/Stemmer;)V", stemmer.jobject)
+
+    @property
+    def stopwords(self):
+        """
+        Returns the stopwords handler.
+        :return: the stopwords handler
+        :rtype: Stopwords
+        """
+        return Stopwords(
+            jobject=javabridge.call(self.jobject, "getStopwordsHandler", "()Lweka/core/stopwords/StopwordsHandler;"))
+
+    @stopwords.setter
+    def stopwords(self, stopwords):
+        """
+        Sets the stopwords handler.
+        :param stopwords: the stopwords handler to use
+        :type stopwords: Stopwords
+        """
+        javabridge.call(
+            self.jobject, "setStopwordsHandler", "(Lweka/core/stopwords/StopwordsHandler;)V", stopwords.jobject)
+
+    @property
+    def tokenizer(self):
+        """
+        Returns the tokenizer.
+        :return: the tokenizer
+        :rtype: Tokenizer
+        """
+        return Tokenizer(
+            jobject=javabridge.call(self.jobject, "getTokenizer", "()Lweka/core/tokenizers/Tokenizer;"))
+
+    @tokenizer.setter
+    def tokenizer(self, tokenizer):
+        """
+        Sets the tokenizer.
+        :param tokenizer: the tokenizer to use
+        :type tokenizer: Tokenizer
+        """
+        javabridge.call(
+            self.jobject, "setTokenizer", "(Lweka/core/tokenizers/Tokenizer;)V", tokenizer.jobject)
+
+
 def main():
     """
     Runs a filter from the command-line. Calls JVM start/stop automatically.
@@ -208,7 +315,7 @@ def main():
 
     jvm.start(jars, max_heap_size=parsed.heap, packages=True)
 
-    logger.debug("Commandline: " + utils.join_options(sys.argv[1:]))
+    logger.debug("Commandline: " + join_options(sys.argv[1:]))
 
     try:
         flter = Filter(parsed.filter)
